function outL = gmm_segmentation(I,inL)
% GMM_SEGMENTATION segment image using independent GMM classifiers.
%
% Synopsis:
%   outL = gmm_segmentation(I,inL)
%
% Input:
%  I [H x W x 3 (uint8)] RGB image of size H x W pixels.
%  inL [H x W (uint8)] Partial image annotation:
%    0 .. pixel not annotated
%    1 .. pixel assigned label 1
%    2 .. pixel assigned label 2
%
% Output:
%  outL [H x W] Complete image segmentation. Each pixel must be 
%    either 1 or  2.
%

% numbe of Gaussian components used for modeling color
K = 5;

imH = size(I,1);
imW = size(I,2);

hard_idx1=find(inL==1);
hard_idx2=find(inL==2);

I=double(I);
Ir=I(:,:,1);
Ig=I(:,:,2);
Ib=I(:,:,3);

X1 = [Ir(hard_idx1) Ig(hard_idx1) Ib(hard_idx1)]';
X2 = [Ir(hard_idx2) Ig(hard_idx2) Ib(hard_idx2)]';
X = [Ir(:) Ig(:) Ib(:)]';
        
[centers,labels,objval] = k_means(X1,K);
model1 = gmm_ml(X1,labels);

[centers,labels,objval] = k_means(X2,K);
model2 = gmm_ml(X2, labels);
        
p1 = -logsumexp(gmm_logpxy(X,model1));
p2 = -logsumexp(gmm_logpxy(X,model2));

%
p1(hard_idx1) = -10000;
p2(hard_idx2) = -10000;

[dummy,outL] = max([p2 ; p1]);
outL = reshape(outL,imH,imW);
  
return;
